@@ -22,6 +22,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "exchange-mapi-connection.h"
 #include "exchange-mapi-folder.h"
 #include "exchange-mapi-utils.h"
@@ -34,8 +38,8 @@ static struct mapi_session *global_mapi_session= NULL;
 static GStaticRecMutex connect_lock = G_STATIC_REC_MUTEX_INIT;
 
 
-#define LOCK() 		g_message("%s(%d): %s: lock(connect_lock)", __FILE__, __LINE__, __PRETTY_FUNCTION__);g_static_rec_mutex_lock(&connect_lock);
-#define UNLOCK() 	g_message("%s(%d): %s: unlock(connect_lock)", __FILE__, __LINE__, __PRETTY_FUNCTION__);g_static_rec_mutex_unlock(&connect_lock);
+#define LOCK() 		g_message("%s: %s: lock(connect_lock)", G_STRLOC, G_STRFUNC);g_static_rec_mutex_lock(&connect_lock);
+#define UNLOCK() 	g_message("%s: %s: unlock(connect_lock)", G_STRLOC, G_STRFUNC);g_static_rec_mutex_unlock(&connect_lock);
 
 #if 0
 #define LOGALL() 	lp_set_cmdline(global_mapi_ctx->lp_ctx, "log level", "10"); global_mapi_ctx->dumpdata = TRUE;
@@ -68,7 +72,7 @@ mapi_profile_load (const char *profname, const char *password)
 	gchar *profpath = NULL;
 	const char *profile = NULL;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
 	if (!g_file_test (profpath, G_FILE_TEST_EXISTS)) {
@@ -82,7 +86,7 @@ mapi_profile_load (const char *profname, const char *password)
 	if (retval != MAPI_E_SUCCESS) {
 		mapi_errstr("MAPIInitialize", GetLastError());
 		if (retval == MAPI_E_SESSION_LIMIT)
-			g_print("\n%s(%d): %s: Already connected ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_print("\n%s: %s: Already connected ", G_STRLOC, G_STRFUNC);
 		goto cleanup;
 	}
 
@@ -109,7 +113,7 @@ cleanup:
 		MAPIUninitialize ();
 	g_free (profpath);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return session;
 }
@@ -129,9 +133,9 @@ exchange_mapi_connection_new (const char *profile, const char *password)
 	UNLOCK();
 
 	if (!global_mapi_session)
-		g_warning ("\n%s(%d): %s: Login failed ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+		g_warning ("\n%s: %s: Login failed ", G_STRLOC, G_STRFUNC);
 	else
-		g_message ("\n%s(%d): %s: Connected ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+		g_message ("\n%s: %s: Connected ", G_STRLOC, G_STRFUNC);
 
 	return global_mapi_session != NULL;
 }
@@ -165,7 +169,7 @@ exchange_mapi_util_read_generic_stream (mapi_object_t *obj_message, uint32_t pro
 	/* if compressed RTF stream, then return */
 	g_return_val_if_fail (proptag != PR_RTF_COMPRESSED, FALSE);
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 	d(g_print("\nAttempt to read stream for proptag 0x%08X ", proptag));
 
 	mem_ctx = talloc_init ("ExchangeMAPI_ReadGenericStream");
@@ -233,7 +237,7 @@ cleanup:
 	mapi_object_release(&obj_stream);
 	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return (retval == MAPI_E_SUCCESS);
 }
@@ -255,7 +259,7 @@ exchange_mapi_util_read_body_stream (mapi_object_t *obj_message, GSList **stream
 	/* sanity check */
 	g_return_val_if_fail (obj_message, FALSE);
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	mem_ctx = talloc_init ("ExchangeMAPI_ReadBodyStream");
 
@@ -373,7 +377,7 @@ exchange_mapi_util_read_body_stream (mapi_object_t *obj_message, GSList **stream
 
 	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return (retval == MAPI_E_SUCCESS);
 }
@@ -388,7 +392,7 @@ exchange_mapi_util_write_generic_streams (mapi_object_t *obj_message, GSList *st
 	enum MAPISTATUS	retval;
 	gboolean 	status = TRUE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 //	mem_ctx = talloc_init ("ExchangeMAPI_WriteGenericStreams");
 
@@ -455,7 +459,7 @@ exchange_mapi_util_write_generic_streams (mapi_object_t *obj_message, GSList *st
 
 //	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return status;
 }
@@ -472,7 +476,7 @@ exchange_mapi_util_delete_attachments (mapi_object_t *obj_message)
 	uint32_t		i_row_attach;
 	gboolean 		status = TRUE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	mem_ctx = talloc_init ("ExchangeMAPI_DeleteAttachments");
 
@@ -528,7 +532,7 @@ cleanup:
 	mapi_object_release(&obj_tb_attach);
 	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return status;
 }
@@ -542,7 +546,7 @@ exchange_mapi_util_set_attachments (mapi_object_t *obj_message, GSList *attach_l
 	enum MAPISTATUS	retval;
 	gboolean 	status = TRUE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	if (remove_existing)
 		exchange_mapi_util_delete_attachments (obj_message);
@@ -587,7 +591,7 @@ exchange_mapi_util_set_attachments (mapi_object_t *obj_message, GSList *attach_l
 
 //	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return status;
 }
@@ -605,7 +609,7 @@ exchange_mapi_util_get_attachments (mapi_object_t *obj_message, GSList **attach_
 	uint32_t		i_row_attach;
 	gboolean 		status = TRUE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	mem_ctx = talloc_init ("ExchangeMAPI_GetAttachments");
 
@@ -698,7 +702,7 @@ cleanup:
 	mapi_object_release(&obj_tb_attach);
 	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return status;
 }
@@ -714,7 +718,7 @@ exchange_mapi_util_get_recipients (mapi_object_t *obj_message, GSList **recip_li
 	uint32_t		i_row_recip;
 	gboolean 		status = TRUE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 //	mem_ctx = talloc_init ("ExchangeMAPI_GetRecipients");
 
@@ -737,7 +741,7 @@ exchange_mapi_util_get_recipients (mapi_object_t *obj_message, GSList **recip_li
 		}
 		/* fail */
 		if (!recipient->email_id) {
-			g_warning ("\n%s:%d %s() - object has a recipient without a PR_SMTP_ADDRESS ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_warning ("\n%s: %s() - object has a recipient without a PR_SMTP_ADDRESS ", G_STRLOC, G_STRFUNC);
 			mapidump_SRow (&(rows_recip.aRow[i_row_recip]), " ");
 		}
 
@@ -752,7 +756,7 @@ cleanup:
 		status = FALSE;
 //	talloc_free (mem_ctx);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return status;
 }
@@ -801,7 +805,7 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 	const char 		**users = NULL;
 	uint32_t 		i, j, count = 0;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	SPropTagArray = set_SPropTagArray(mem_ctx, 0x6,
 					  PR_OBJECT_TYPE,
@@ -839,7 +843,7 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 		if (FlagList->aulPropTag[i] == MAPI_AMBIGUOUS) {
 			/* We should never get an ambiguous resolution as we use the email-id for resolving. 
 			 * However, if we do still get an ambiguous entry, we can't handle it :-( */
-			g_warning ("\n%s:%d %s() - '%s' is ambiguous ", __FILE__, __LINE__, __PRETTY_FUNCTION__, recipient->email_id);
+			g_warning ("\n%s: %s() - '%s' is ambiguous ", G_STRLOC, G_STRFUNC, recipient->email_id);
 		} else if (FlagList->aulPropTag[i] == MAPI_UNRESOLVED) {
 			/* If the recipient is unresolved, consider it is a SMTP one */
 			SRowSet->aRow = talloc_realloc(mem_ctx, SRowSet->aRow, struct SRow, SRowSet->cRows + 1);
@@ -872,7 +876,7 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 cleanup:
 	g_free (users);
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 }
 
 GSList *
@@ -888,7 +892,7 @@ exchange_mapi_util_check_restriction (mapi_id_t fid, struct mapi_SRestriction *r
 	uint32_t count, i;
 	GSList *mids = NULL;
 
-	d(g_print("\n%s(%d): Entering %s: folder-id %016llX ", __FILE__, __LINE__, __PRETTY_FUNCTION__, fid));
+	d(g_print("\n%s: Entering %s: folder-id %016" G_GINT64_MODIFIER "X ", G_STRLOC, G_STRFUNC, fid));
 
 	LOCK();
 	LOGALL();
@@ -980,7 +984,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return mids;
 }
@@ -1003,7 +1007,7 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 	uint32_t count, i, cursor_pos = 0;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s: folder-id %016llX ", __FILE__, __LINE__, __PRETTY_FUNCTION__, fid));
+	d(g_print("\n%s: Entering %s: folder-id %016" G_GINT64_MODIFIER "X ", G_STRLOC, G_STRFUNC, fid));
 
 	LOCK();
 	LOGALL();
@@ -1096,8 +1100,8 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 			/* Add named props using callback */
 			if (build_name_id) {
 				if (!build_name_id (nameid, build_name_data)) {
-					g_warning ("\n%s(%d): (%s): Could not build named props ", __FILE__, 
-						   __LINE__, __PRETTY_FUNCTION__);
+					g_warning ("\n%s: (%s): Could not build named props ",
+							G_STRLOC, G_STRFUNC);
 					goto GetProps_cleanup;
 				}
 
@@ -1162,7 +1166,6 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 				struct SPropValue *lpProps;
 				uint32_t prop_count = 0, k;
 
-				lpProps = talloc_zero(mem_ctx, struct SPropValue);
 				retval = GetProps (&obj_message, GetPropsTagArray, &lpProps, &prop_count);
 
 				/* Conversion from SPropValue to mapi_SPropValue. (no padding here) */
@@ -1227,7 +1230,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s: folder-id %016llX ", __FILE__, __LINE__, __PRETTY_FUNCTION__, fid));
+	d(g_print("\n%s: Leaving %s: folder-id %016" G_GINT64_MODIFIER "X ", G_STRLOC, G_STRFUNC, fid));
 
 	return result;
 }
@@ -1251,7 +1254,8 @@ exchange_mapi_connection_fetch_item (mapi_id_t fid, mapi_id_t mid,
 	GSList *stream_list = NULL;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s: folder-id %016llX message-id %016llX ", __FILE__, __LINE__, __PRETTY_FUNCTION__, fid, mid));
+	d(g_print("\n%s: Entering %s: folder-id %016" G_GINT64_MODIFIER "X message-id %016" G_GINT64_MODIFIER "X",
+				G_STRLOC, G_STRFUNC, fid, mid));
 
 	LOCK();
 	LOGALL();
@@ -1290,7 +1294,7 @@ exchange_mapi_connection_fetch_item (mapi_id_t fid, mapi_id_t mid,
 		/* Add named props using callback */
 		if (build_name_id) {
 			if (!build_name_id (nameid, build_name_data)) {
-				g_warning ("\n%s(%d): (%s): Could not build named props ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+				g_warning ("\n%s: (%s): Could not build named props ", G_STRLOC, G_STRFUNC);
 				goto GetProps_cleanup;
 			}
 
@@ -1398,7 +1402,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -1414,7 +1418,7 @@ exchange_mapi_create_folder (uint32_t olFolder, mapi_id_t pfid, const char *name
 	const char *type;
 	mapi_id_t fid = 0;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1472,7 +1476,7 @@ exchange_mapi_create_folder (uint32_t olFolder, mapi_id_t pfid, const char *name
 	}
 
 	fid = mapi_object_get_id (&obj_folder);
-	g_print("\nFolder %s created with id %016llX ", name, fid);
+	g_print("\nFolder %s created with id %016" G_GINT64_MODIFIER "X ", name, fid);
 
 cleanup:
 	mapi_object_release(&obj_folder);
@@ -1481,7 +1485,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	/* Shouldn't we return (ExchangeMAPIFolder *) instead of a plain fid ? */
 	return fid;
@@ -1495,7 +1499,7 @@ exchange_mapi_empty_folder (mapi_id_t fid)
 	mapi_object_t obj_folder;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1522,7 +1526,7 @@ exchange_mapi_empty_folder (mapi_id_t fid)
 		goto cleanup;
 	}
 
-	g_print("\nFolder with id %016llX was emptied ", fid);
+	g_print("\nFolder with id %016" G_GINT64_MODIFIER "X was emptied ", fid);
 
 	result = TRUE;
 
@@ -1532,7 +1536,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -1548,7 +1552,7 @@ exchange_mapi_remove_folder (uint32_t olFolder, mapi_id_t fid)
 	ExchangeMAPIFolder *folder;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	folder = exchange_mapi_folder_get_folder (fid);
 	g_return_val_if_fail (folder != NULL, FALSE);
@@ -1583,7 +1587,7 @@ exchange_mapi_remove_folder (uint32_t olFolder, mapi_id_t fid)
 		goto cleanup;
 	}
 
-	g_print("\nFolder with id %016llX was emptied ", fid);
+	g_print("\nFolder with id %016" G_GINT64_MODIFIER "X was emptied ", fid);
 
 	/* Attempt to open the top/parent folder */
 	retval = OpenFolder(&obj_store, folder->parent_folder_id, &obj_top);
@@ -1599,7 +1603,7 @@ exchange_mapi_remove_folder (uint32_t olFolder, mapi_id_t fid)
 		goto cleanup;
 	}
 
-	g_print("\nFolder with id %016llX was deleted ", fid);
+	g_print("\nFolder with id %016" G_GINT64_MODIFIER "X was deleted ", fid);
 
 	result = TRUE;
 
@@ -1610,7 +1614,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -1625,7 +1629,7 @@ exchange_mapi_rename_folder (mapi_id_t fid, const char *new_name)
 	TALLOC_CTX *mem_ctx;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1664,7 +1668,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -1681,7 +1685,7 @@ exchange_mapi_util_resolve_named_props (uint32_t olFolder, mapi_id_t fid,
 	struct SPropTagArray *SPropTagArray, *ret_array = NULL;
 	uint32_t i;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1718,7 +1722,7 @@ exchange_mapi_util_resolve_named_props (uint32_t olFolder, mapi_id_t fid,
 	/* Add named props using callback */
 	if (build_name_id) {
 		if (!build_name_id (nameid, ni_data)) {
-			g_warning ("\n%s(%d): (%s): Could not build named props ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_warning ("\n%s: (%s): Could not build named props ", G_STRLOC, G_STRFUNC);
 			goto cleanup;
 		}
 
@@ -1742,7 +1746,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return ret_array;
 }
@@ -1758,7 +1762,7 @@ exchange_mapi_util_resolve_named_prop (uint32_t olFolder, mapi_id_t fid, uint16_
 	struct SPropTagArray *SPropTagArray, *ret_array = NULL;
 	uint32_t i;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1813,7 +1817,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return ret_array;
 }
@@ -1831,7 +1835,7 @@ exchange_mapi_util_create_named_prop (uint32_t olFolder, mapi_id_t fid,
 	struct SPropTagArray *SPropTagArray;
 	uint32_t propID = 0x00000000;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1887,7 +1891,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return propID; 
 }
@@ -1899,7 +1903,7 @@ exchange_mapi_get_default_folder_id (uint32_t olFolder)
 	mapi_object_t obj_store;
 	mapi_id_t fid = 0;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1923,7 +1927,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return (retval == MAPI_E_SUCCESS ? fid : 0);
 }
@@ -1946,7 +1950,7 @@ exchange_mapi_create_item (uint32_t olFolder, mapi_id_t fid,
 	gint propslen = 0;
 	mapi_id_t mid = 0;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -1993,7 +1997,7 @@ exchange_mapi_create_item (uint32_t olFolder, mapi_id_t fid,
 	/* Add named props using callback */
 	if (build_name_id) {
 		if (!build_name_id (nameid, ni_data)) {
-			g_warning ("\n%s(%d): (%s): Could not build named props ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_warning ("\n%s: (%s): Could not build named props ", G_STRLOC, G_STRFUNC);
 			goto cleanup;
 		}
 
@@ -2008,7 +2012,7 @@ exchange_mapi_create_item (uint32_t olFolder, mapi_id_t fid,
 	if (build_props) {
 		propslen = build_props (&props, SPropTagArray, p_data);
 		if (propslen < 1) {
-			g_warning ("\n%s(%d): (%s): build_props failed! propslen = %d ", __FILE__, __LINE__, __PRETTY_FUNCTION__, propslen);
+			g_warning ("\n%s: (%s): build_props failed! propslen = %d ", G_STRLOC, G_STRFUNC, propslen);
 			goto cleanup;
 		}
 	}
@@ -2060,7 +2064,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return mid;
 }
@@ -2083,7 +2087,7 @@ exchange_mapi_modify_item (uint32_t olFolder, mapi_id_t fid, mapi_id_t mid,
 	gint propslen = 0;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2130,7 +2134,7 @@ exchange_mapi_modify_item (uint32_t olFolder, mapi_id_t fid, mapi_id_t mid,
 	/* Add named props using callback */
 	if (build_name_id) {
 		if (!build_name_id (nameid, ni_data)) {
-			g_warning ("\n%s(%d): (%s): Could not build named props ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_warning ("\n%s: (%s): Could not build named props ", G_STRLOC, G_STRFUNC);
 			goto cleanup;
 		}
 
@@ -2145,7 +2149,8 @@ exchange_mapi_modify_item (uint32_t olFolder, mapi_id_t fid, mapi_id_t mid,
 	if (build_props) {
 		propslen = build_props (&props, SPropTagArray, p_data);
 		if (propslen < 1) {
-			g_warning ("\n%s(%d): (%s): Could not build props ", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+			g_warning ("\n%s: (%s): Could not build props ",
+					G_STRLOC, G_STRFUNC);
 			goto cleanup;
 		}
 	}
@@ -2197,7 +2202,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -2214,7 +2219,7 @@ exchange_mapi_set_flags (uint32_t olFolder, mapi_id_t fid, GSList *mids, uint32_
 	GSList *tmp = mids;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2256,7 +2261,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -2320,7 +2325,7 @@ exchange_mapi_copy_items (mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids)
 {
 	gboolean result = FALSE; 
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2328,7 +2333,7 @@ exchange_mapi_copy_items (mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids)
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result; 
 }
@@ -2338,7 +2343,7 @@ exchange_mapi_move_items (mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids)
 {
 	gboolean result = FALSE; 
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2346,7 +2351,7 @@ exchange_mapi_move_items (mapi_id_t src_fid, mapi_id_t dest_fid, GSList *mids)
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result; 
 }
@@ -2363,7 +2368,7 @@ exchange_mapi_remove_items (uint32_t olFolder, mapi_id_t fid, GSList *mids)
 	GSList *tmp = mids;
 	gboolean result = FALSE;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2416,7 +2421,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -2489,7 +2494,7 @@ get_child_folders(TALLOC_CTX *mem_ctx, ExchangeMAPIFolderCategory folder_hier, m
 			class = IPF_NOTE;
 
 		newname = utf8tolinux (name);
-		g_print("\n|---+ %-15s : (Container class: %s %016llX) UnRead : %d Total : %d ", newname, class, *fid, unread ? *unread : 0, total ? *total : 0);
+		g_print("\n|---+ %-15s : (Container class: %s %016" G_GINT64_MODIFIER "X) UnRead : %d Total : %d ", newname, class, *fid, unread ? *unread : 0, total ? *total : 0);
 
 		folder = exchange_mapi_folder_new (newname, class, folder_hier, *fid, folder_id, child ? *child : 0, unread ? *unread : 0, total ? *total : 0);
 		*mapi_folders = g_slist_prepend (*mapi_folders, folder);
@@ -2508,21 +2513,122 @@ cleanup:
 	return result;
 }
 
+/* TODO : Find a right place for this. */
+#define PR_ADDITIONAL_REN_ENTRYIDS    PROP_TAG(PT_MV_BINARY, 0x36D8)
+
+/*NOTE : This should be called when you hold the connection lock*/
+/*NOTE : IsMailboxFolder doesn't support this yet. */
+/* Ticket : http://trac.openchange.org/ticket/134  */
+static void
+mapi_get_ren_additional_fids (mapi_object_t *obj_store, GHashTable **folder_list)
+{
+	mapi_id_t inbox_id, fid;
+	mapi_object_t obj_folder_inbox;
+	struct SPropTagArray *SPropTagArray;
+	struct SPropValue *lpProps;
+	struct SRow aRow;
+	const struct BinaryArray_r *entryids;
+	struct Binary_r entryid;
+	enum MAPISTATUS retval;
+
+	guint32 count, *folder_type;
+	guint i = 0;
+
+	TALLOC_CTX *mem_ctx;
+
+	/*Note : Do not change the order.*/
+	const guint32 olfolder_defaults[] = {
+		olFolderConflicts,
+		olFolderSyncIssues,
+		olFolderLocalFailures,
+		olFolderServerFailures,
+		olFolderJunk
+	};
+
+	mem_ctx = talloc_init("ExchangeMAPI_GetAdditionalFIDs");
+	mapi_object_init(&obj_folder_inbox);
+
+	/* Get Inbox FID using GetDefaultFolder. */
+	retval = GetDefaultFolder(obj_store, &inbox_id, olFolderInbox);
+	if (retval != MAPI_E_SUCCESS) {
+		mapi_errstr("GetDefaultFolder", GetLastError());
+		goto cleanup;
+	}
+
+	/* Open InboxFolder. */
+	retval = OpenFolder(obj_store, inbox_id, &obj_folder_inbox);
+	if (retval != MAPI_E_SUCCESS) {
+		mapi_errstr("OpenFolder", GetLastError());
+		goto cleanup;
+	}
+
+	/* GetProps on Inbox for PR_ADDITIONAL_REN_ENTRYIDS */
+	SPropTagArray = set_SPropTagArray(mem_ctx, 0x1, PR_ADDITIONAL_REN_ENTRYIDS);
+
+	lpProps = talloc_zero(mem_ctx, struct SPropValue);
+	retval = GetProps (&obj_folder_inbox, SPropTagArray, &lpProps, &count);
+
+	/* Build a SRow structure */
+	aRow.ulAdrEntryPad = 0;
+	aRow.cValues = count;
+	aRow.lpProps = lpProps;
+
+	entryids = (const struct BinaryArray_r *) find_SPropValue_data(&aRow, PR_ADDITIONAL_REN_ENTRYIDS);
+
+	/* Iterate through MV_BINARY */
+	if (entryids) {
+		for (i = 0; i < G_N_ELEMENTS (olfolder_defaults); i++) {
+			entryid = entryids->lpbin [i];
+			retval = GetFIDFromEntryID(entryid.cb, entryid.lpb, inbox_id, &fid);
+
+			if (fid) {
+				folder_type = g_new0 (guint32, 1);
+				*folder_type = olfolder_defaults[i];
+
+				g_hash_table_insert (*folder_list, 
+						     exchange_mapi_util_mapi_id_to_string (fid), 
+						     folder_type);
+			}
+		}
+	}
+
+cleanup:
+	mapi_object_release(&obj_folder_inbox);
+	talloc_free (mem_ctx);
+}
+
 static void 
 set_default_folders (mapi_object_t *obj_store, GSList **mapi_folders)
 {
 	GSList *folder_list = *mapi_folders;
+
+	GHashTable *default_folders = g_hash_table_new_full (g_str_hash, g_str_equal,
+							     g_free, g_free);
+
+	mapi_get_ren_additional_fids (obj_store, &default_folders);
 	
 	while (folder_list != NULL) {
 		ExchangeMAPIFolder *folder = NULL;
 		guint32 default_type = 0;
+		gchar *key_fid = NULL;
+		gpointer value = NULL;
+
 		folder = folder_list->data;
-		if (IsMailboxFolder (obj_store,folder->folder_id, &default_type )) {
+		key_fid = exchange_mapi_util_mapi_id_to_string (folder->folder_id);
+
+		if ((value = g_hash_table_lookup (default_folders, key_fid)) != NULL)
+			default_type = *(guint32 *)value;
+		g_free (key_fid);
+
+		if (default_type != 0 || IsMailboxFolder (obj_store,folder->folder_id, &default_type)) {
 			folder->is_default = true; /* TODO : Clean up. Redundant.*/
 			folder->default_type = default_type;
 		}
+
 		folder_list = g_slist_next (folder_list); 
 	}
+
+	g_hash_table_destroy (default_folders);
 }
 
 static void 
@@ -2557,7 +2663,7 @@ exchange_mapi_get_folders_list (GSList **mapi_folders)
 	const char 		*mailbox_owner_name = NULL;
 	const char 		*mailbox_user_name = NULL;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2606,7 +2712,11 @@ exchange_mapi_get_folders_list (GSList **mapi_folders)
 	utf8_mailbox_name = utf8tolinux (mailbox_name);
 
 	/* FIXME: May have to get the child folders count? Do we need/use it? */
-	folder = exchange_mapi_folder_new (utf8_mailbox_name, IPF_NOTE, MAPI_PERSONAL_FOLDER, mailbox_id, 0, 0, 0 ,0); 
+	folder = exchange_mapi_folder_new (utf8_mailbox_name, IPF_NOTE, 
+					   MAPI_PERSONAL_FOLDER, mailbox_id, 0, 0, 0 ,0); 
+	folder->is_default = true;
+	folder->default_type = olFolderTopInformationStore; /*Is this correct ?*/
+
 	*mapi_folders = g_slist_prepend (*mapi_folders, folder);
 
 	/* FIXME: check status of get_child_folders */
@@ -2628,7 +2738,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -2643,7 +2753,7 @@ exchange_mapi_get_pf_folders_list (GSList **mapi_folders)
 	mapi_id_t		mailbox_id;
 	ExchangeMAPIFolder 	*folder;
 
-	d(g_print("\n%s(%d): Entering %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
 	LOCK();
 	LOGALL();
@@ -2665,7 +2775,10 @@ exchange_mapi_get_pf_folders_list (GSList **mapi_folders)
 	}
 
 	/*  TODO : Localized string */
-	folder = exchange_mapi_folder_new ("All Public Folders", IPF_NOTE, 0, mailbox_id, 0, 0, 0 ,0);
+	folder = exchange_mapi_folder_new ("All Public Folders", IPF_NOTE, 0, 
+					   mailbox_id, 0, 0, 0 ,0);
+	folder->is_default = true;
+	folder->default_type = olPublicFoldersAllPublicFolders;
 
 	*mapi_folders = g_slist_prepend (*mapi_folders, folder);
 
@@ -2680,7 +2793,7 @@ cleanup:
 	LOGNONE();
 	UNLOCK();
 
-	d(g_print("\n%s(%d): Leaving %s ", __FILE__, __LINE__, __PRETTY_FUNCTION__));
+	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return result;
 }
@@ -2730,3 +2843,168 @@ exchange_mapi_util_ex_to_smtp (const gchar *ex_address)
 
 	return smtp_addr;
 }
+
+/* Shows error message on the console, and, if error_msg is not NULL, then
+   sets it to the similar error message as well. */
+static void
+manage_mapi_error (const char *context, uint32_t error_id, char **error_msg)
+{
+	if (!context)
+		context = "???";
+
+	mapi_errstr (context, error_id);
+
+	if (error_msg) {
+		char *e = g_strconcat (context, ":", mapi_get_errstr (error_id), NULL);
+
+		g_free (*error_msg);
+		*error_msg = e;
+	}
+}
+
+gboolean
+exchange_mapi_create_profile (const char *username, const char *password, const char *domain, const char *server, char **error_msg)
+{
+	enum MAPISTATUS	retval;
+	gboolean result = FALSE; 
+	const gchar *workstation = "localhost";
+	gchar *profname = NULL, *profpath = NULL;
+	struct mapi_session *session = NULL;
+
+	/*We need all the params before proceeding.*/
+	g_return_val_if_fail (username && *username && password && *password &&
+			      domain && *domain && server && *server, FALSE);
+
+	d(g_print ("Create profile with %s %s %s\n", username, domain, server));
+
+	LOCK ();
+
+	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
+	profname = g_strdup_printf("%s@%s", username, domain);
+
+	if (!g_file_test (profpath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
+		/* Create a ProfileStore */
+		retval = CreateProfileStore (profpath, LIBMAPI_LDIF_DIR); 
+		if (retval != MAPI_E_SUCCESS) {
+			manage_mapi_error ("CreateProfileStore", GetLastError(), error_msg);
+			goto cleanup;
+		}
+	}
+
+	retval = MAPIInitialize(profpath); 
+	if (retval == MAPI_E_SESSION_LIMIT)
+	/* do nothing, the profile store is already initialized */
+		manage_mapi_error ("MAPIInitialize", GetLastError(), error_msg); 
+	else if (retval != MAPI_E_SUCCESS) {
+		manage_mapi_error ("MAPIInitialize", GetLastError(), error_msg);
+		goto cleanup; 
+	}
+
+	/* Delete any existing profiles with the same profilename */
+	retval = DeleteProfile(profname); 
+	/* don't bother to check error - it would be valid if we got an error */
+
+	retval = CreateProfile(profname, username, password, 0); 
+	if (retval != MAPI_E_SUCCESS) {
+		manage_mapi_error ("CreateProfile", GetLastError(), error_msg);
+		goto cleanup; 
+	}
+
+	mapi_profile_add_string_attr(profname, "binding", server);
+	mapi_profile_add_string_attr(profname, "workstation", workstation);
+	mapi_profile_add_string_attr(profname, "domain", domain);
+	
+	/* This is only convenient here and should be replaced at some point */
+	mapi_profile_add_string_attr(profname, "codepage", "0x4e4");
+	mapi_profile_add_string_attr(profname, "language", "0x409");
+	mapi_profile_add_string_attr(profname, "method", "0x409");
+	
+	/* Login now */
+	d(g_print("Logging into the server... "));
+	retval = MapiLogonProvider(&session, profname, password, PROVIDER_ID_NSPI); 
+	if (retval != MAPI_E_SUCCESS) {
+		manage_mapi_error ("MapiLogonProvider", GetLastError(), error_msg);
+		g_debug ("Deleting profile %s ", profname); 
+		retval = DeleteProfile(profname); 
+		if (retval != MAPI_E_SUCCESS)
+			manage_mapi_error ("DeleteProfile", GetLastError(), error_msg);
+		goto cleanup; 
+	}
+	d(g_print("succeeded \n"));
+
+	retval = ProcessNetworkProfile(session, username, NULL, NULL); 
+	if (retval != MAPI_E_SUCCESS) {
+		manage_mapi_error ("ProcessNetworkProfile", GetLastError(), error_msg);
+		goto cleanup; 
+	}
+
+	/* Set it as the default profile. Is this needed? */
+	retval = SetDefaultProfile(profname); 
+	if (retval != MAPI_E_SUCCESS) {
+		manage_mapi_error ("SetDefaultProfile", GetLastError(), error_msg);
+		goto cleanup; 
+	}
+
+	/* Close the connection, so that we can login with what we created */
+	exchange_mapi_connection_close ();
+
+	/* Initialize a global connection */
+	if (exchange_mapi_connection_new (profname, password)) {
+		result = TRUE;
+		exchange_mapi_peek_folder_list ();
+	}
+
+cleanup: 
+	if (!result)
+		MAPIUninitialize ();
+
+	g_free (profname);
+	g_free (profpath);
+
+	UNLOCK ();
+
+	return result;
+}
+
+gboolean
+exchange_mapi_delete_profile (const char *profile)
+{
+	enum MAPISTATUS	retval;
+	gboolean result = FALSE; 
+	gchar *profpath = NULL;
+
+	LOCK ();
+
+	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
+	if (!g_file_test (profpath, G_FILE_TEST_EXISTS)) {
+		g_warning ("No need to delete profile. DB itself is missing \n");
+		result = TRUE;
+		goto cleanup; 
+	}
+
+	retval = MAPIInitialize(profpath); 
+	if (retval == MAPI_E_SESSION_LIMIT)
+	/* do nothing, the profile store is already initialized */
+		; 
+	else if (retval != MAPI_E_SUCCESS) {
+		mapi_errstr("MAPIInitialize", GetLastError());
+		goto cleanup; 
+	}
+
+	g_debug ("Deleting profile %s ", profile); 
+	retval = DeleteProfile(profile); 
+	if (retval != MAPI_E_SUCCESS) {
+		mapi_errstr("DeleteProfile", GetLastError());
+		goto cleanup; 
+	}
+
+	exchange_mapi_connection_close ();
+	result = TRUE; 
+
+cleanup: 
+	g_free(profpath);
+
+	UNLOCK ();
+
+	return result;
+}
